#define UNICODE
#define _UNICODE

#include "windows.h"
#include "auth.h"

BOOL CALLBACK AuthenticateDlgProc(HWND, UINT, WPARAM, LPARAM);

BOOL AuthenticateUser(HINSTANCE hInstance, LPTSTR pszFile)
{
    TCHAR       sz[MAX_PATH], szRemote[MAX_PATH], szDrive[8];
    DWORD       i, status, count, size;
    LPTSTR      p;
    AUTH_INFO   authInfo;
    NETRESOURCE NetResource;
    NETRESOURCE*pNetRes;
    BYTE        Data[1024];
    HANDLE      hEnum;
    BOOL        match;

    if (pszFile == NULL) {
        return FALSE;
    }
    
    // Remote path should be in one of these two forms:
    //         \\server\share\...\filename
    //         x:\...\filename
    //
    // In either case, we need \\server\share corresponding to
    // remote drive.

    lstrcpy(szRemote, pszFile);
    if (lstrlen(szRemote) < 2) {
        return FALSE;               // poorly formatted file name
    }

    if (szRemote[0] == TEXT('\\') && szRemote[1] == TEXT('\\')) {

        // Starts with a server name, truncate after share name.

        for (p = &szRemote[2]; *p != 0; p++) {
            // Skip to the share name.
            if (*p == TEXT('\\')) {
                // Truncate path after the share name.
                p++;
                while ((*p != 0) && (*p != TEXT('\\'))) p++;
                if (*p == TEXT('\\')) *p = 0;
            }
        }

    } else if (szRemote[1] == TEXT(':')) {
        // Starts with a mapped drive letter, must be a
        // disconnected or unavailable drive letter mapping.
        // Retrieve the server and share this drive letter was
        // mapped to.

        count = 0xFFFFFFFF;
        size = 1024;
        // copy x: and terminating null
        lstrcpyn(szDrive, szRemote, 3);
        status = WNetOpenEnum(RESOURCE_REMEMBERED, 
                              RESOURCETYPE_DISK,
                              0, NULL, &hEnum);
        if (status != NO_ERROR) {
            return status;
        }

        status = WNetEnumResource(hEnum, &count, Data, &size);
        if (status != NO_ERROR) {
            WNetCloseEnum(hEnum);
            return status;
        }
 
        pNetRes = (NETRESOURCE *)Data;
        i = 0;
        match = FALSE;
        while (i < count) {
            if (lstrcmpi(pNetRes->lpLocalName, szDrive) == 0) {
                lstrcpy(szRemote, pNetRes->lpRemoteName);
                lstrcpy(sz, szRemote);
                lstrcat(sz, &pszFile[2]);
                lstrcpy(pszFile, sz);   // replace original path
                match = TRUE;
                break;
            }
            i++;
            pNetRes++;
        }
 
        if (!match) {
            return FALSE;
        }

    } else {
        return FALSE;               // poorly formatted file name
    }

    // fill out private AUTH_INFO struct and pass to dialog box.

    authInfo.hInstance = hInstance;
    lstrcpy(authInfo.szFileName, pszFile);

    status = DialogBoxParam(hInstance, 
                            MAKEINTRESOURCE(DLG_AUTHENTICATE),
                            NULL, AuthenticateDlgProc, 
                            (DWORD)&authInfo);
    if (!status) {
        return status;
    }

    // Authenticate (logon) the user

    memset(&NetResource, 0, sizeof(NetResource));

    NetResource.dwType = RESOURCETYPE_DISK; 
    NetResource.lpLocalName = 0;    // no drive mapping 
    NetResource.lpRemoteName = szRemote; 
    NetResource.lpProvider = NULL; 

    // if domain specified, prefix it to username

    if (authInfo.szDomain[0] != 0) {
        wsprintf(sz, TEXT("%s\\%s"), authInfo.szDomain, 
                 authInfo.szUser);
    } else {
        lstrcpy(sz, authInfo.szUser);
    }

    status = WNetAddConnection3(NULL, &NetResource, 
                                authInfo.szPassword, sz, 0); 
    
    if (status != NO_ERROR) {
        wsprintf(sz, TEXT("WNetAddConnection3 Failed (%d)"), 
                 status);
        MessageBox(NULL, sz, TEXT("AUTH.EXE"), MB_OK);
        return status;            
    }

    return TRUE;

} // AuthenticateUser

//-----------------------------------------------------------------
BOOL CALLBACK AuthenticateDlgProc(HWND hDlg, UINT uMsg, 
                                  WPARAM wParam, LPARAM lParam)
{
    TCHAR szRes[MAX_PATH], sz[MAX_PATH];
    static PAUTH_INFO pAuthInfo;

    switch (uMsg) {
    case WM_INITDIALOG:
        pAuthInfo = (PAUTH_INFO)lParam;
        LoadString(pAuthInfo->hInstance, IDS_AUTH_TEXT, szRes,
                   MAX_PATH);
        wsprintf(sz, szRes, pAuthInfo->szFileName);
        SetDlgItemText(hDlg, ID_LABEL, sz);
        SendMessage(GetDlgItem(hDlg, ID_PASSWORD), 
                    EM_SETPASSWORDCHAR, (WPARAM)TEXT('*'), 0); 
        return TRUE;

    case WM_COMMAND:
        switch (LOWORD(wParam)) {
        case IDOK:

            // Read account information.

            GetDlgItemText(hDlg, ID_USER, pAuthInfo->szUser, 
                           MAX_PATH);
            GetDlgItemText(hDlg,ID_PASSWORD, pAuthInfo->szPassword,
                           MAX_PATH);
            GetDlgItemText(hDlg,ID_DOMAIN, pAuthInfo->szDomain, 
                            MAX_PATH);
            EndDialog(hDlg, TRUE);
            return TRUE;

        case IDCANCEL:
            EndDialog(hDlg, FALSE);
            return TRUE;
        }
        break;
    }

    return FALSE;

} // AuthenticateDlgProc
