/*------------------------------------------------------------------
CplFnc.C:    Implementation of CPL Reusable Functions
Author:      Zuoliu Ding
------------------------------------------------------------------*/
#define  STRICT
#include <windows.h>
#include <cpl.h>   

#ifndef WIN32
#include <stdio.h>
#include <malloc.h>
#include <string.h>    
#endif

#include "cplfnc.h"

#define LEN_NAME    32
#define RETICPERR(h,m) {MessageBox(h,m,"ICP Error:", MB_OK); \
                        FreeLibrary(g_InstCpl); \
                        return ECAERR_CPLINIT;}

typedef struct CPLBUF         // for internal use
{
    LONG    lData;            // user defined data 
    HICON   hIcon;            // icon, not used  
    char    szName[LEN_NAME]; // short name 
    char    szInfo[64];       // long name (status line), not used  
} CPLBUF, *LPCPLBUF;

HINSTANCE    g_InstCpl;       // Global CPL Instance
APPLET_PROC  g_proc;          // Global CPL Process Address
LPCPLBUF     g_CplBuf;        // Global CPL Buffer
int          g_nCount;        // Global Applet Number in a CPL

/*/////////////////////////////////////////////////////////////////
Function:    ShowLastError()
Note:        Format the Last Error Message
*//////////////////////////////////////////////////////////////////
VOID ShowLastError(HWND hwnd, DWORD dwErr)
{   
    char szBuf[256];      

#ifdef WIN32    
    FormatMessage(FORMAT_MESSAGE_FROM_SYSTEM, NULL, 
                  dwErr,
                  MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
                  (LPTSTR)szBuf, 256, NULL );
#else
    sprintf(szBuf, "Error Code: %d", (int)g_InstCpl);
#endif

    MessageBox(hwnd, (LPSTR)szBuf, "Load Library Error", MB_OK);
}

/*/////////////////////////////////////////////////////////////////
Function:   CheckUnicode()
Parameters: pszFrom  Applet Name input
            pszTo    Applet Name returned
Note:       Helper for Checking OS Version and Unicode on NT
*//////////////////////////////////////////////////////////////////
void CheckUnicode(LPSTR pszFrom, LPSTR pszTo)
{
#ifdef WIN32
    OSVERSIONINFO os = {sizeof(OSVERSIONINFO)};
    GetVersionEx(&os);
    
    if (os.dwPlatformId ==VER_PLATFORM_WIN32_NT)
    {
        int iFlg = IS_TEXT_UNICODE_STATISTICS;
        IsTextUnicode((LPVOID)pszFrom, LEN_NAME, &iFlg);
        if (iFlg == IS_TEXT_UNICODE_STATISTICS)
        {
            wcstombs(pszTo, (LPWSTR)pszFrom, LEN_NAME); 
        }
    }
#endif

    strcpy(pszTo, pszFrom); // for 16-bit, ANSI, or non-NT
}

/*/////////////////////////////////////////////////////////////////
Function:   GetCPLBuffer()
Parameters: idx      Applet index
Note:       Helper for using CPL_INQUIRE and CPL_NEWINQUIRE
*//////////////////////////////////////////////////////////////////
void GetCPLBuffer(HWND hwnd, int idx)
{
    NEWCPLINFO   newInfo;
    CPLINFO      cplInfo;
    char         szName[LEN_NAME];

    memset(&cplInfo, 0, sizeof(CPLINFO));
    g_proc(hwnd, CPL_INQUIRE, (LONG)idx, (LPARAM)&cplInfo);     
    
    LoadString(g_InstCpl, cplInfo.idName, szName, LEN_NAME);
    g_CplBuf[idx].lData = cplInfo.lData;

    if (!szName[0] || !g_CplBuf[idx].lData)
    {
        memset(&newInfo, 0, sizeof(NEWCPLINFO));
        g_proc(hwnd, CPL_NEWINQUIRE, (LONG)idx, (LPARAM)&newInfo);

        if (!szName[0])                // NULL string 
            strcpy(szName, newInfo.szName);
        if (!g_CplBuf[idx].lData) 
            g_CplBuf[idx].lData = newInfo.lData;
    }

    CheckUnicode(szName, g_CplBuf[idx].szName);
}

/*/////////////////////////////////////////////////////////////////
Function:   InitCPLProc()
Parameters: hwnd    Parent window handle 
            pszCpl  Address to the Name of CPL File
Note:       Global initialization, allocate and fill in CPL buffer
*//////////////////////////////////////////////////////////////////
int InitCPLProc(HWND hwnd, LPCSTR pszCpl)
{
    int i;
    g_InstCpl = LoadLibrary(pszCpl);    

#ifdef WIN32    
    if ((long)g_InstCpl <HINSTANCE_ERROR)    
    {
        DWORD dwErr = GetLastError();
// Err: 31, A device attached to the system is not functioning.
        if (dwErr==ERROR_GEN_FAILURE) 
            return ERROR_GEN_FAILURE; // get out

        ShowLastError(hwnd, dwErr);   // other error
        return ECAERR_CPLINIT; 
    }
#else      // for 16-bit       
    if (g_InstCpl <HINSTANCE_ERROR)   
    {
// Err: 21, Application requires MS Windows 32-bit extensions. 
        if ((int)g_InstCpl==ECAERR_NEED32BIT) 
            return ECAERR_NEED32BIT;  // get out

        ShowLastError(hwnd, 0);       // other error
        return ECAERR_CPLINIT; 
    }
#endif

    // no error for g_InstCpl 
    g_proc =(APPLET_PROC)GetProcAddress(g_InstCpl, "CPlApplet");
    if (!g_proc)
        RETICPERR(hwnd, "Can't Get Proc Addr !");
 
    if (!g_proc(hwnd, CPL_INIT, 0L, 0L)) // INIT Err: return 0
        RETICPERR(hwnd, "Cpl Init Failed !");
    
    if (!(g_nCount = (int)g_proc(hwnd, CPL_GETCOUNT, 0L, 0L)))
        RETICPERR(hwnd, "No Applet in CPL !");

    g_CplBuf = (LPCPLBUF)malloc(g_nCount*sizeof(CPLBUF));
    if (!g_CplBuf)
        RETICPERR(hwnd, "No Memory Allocated !");

    for (i=0; i<g_nCount; ++i) GetCPLBuffer(hwnd, i);

    return ECAERR_NOERROR;
}

/*/////////////////////////////////////////////////////////////////
Function:   CloseCPL()
Parameters: hwnd    Parent window handle
Note:       Helper for Cleanup from a CPL
*//////////////////////////////////////////////////////////////////
void CloseCPL(HWND hwnd)
{
    int i;
    for (i=0; i<g_nCount; ++i)
       g_proc(hwnd, CPL_STOP, (LONG)i, g_CplBuf[i].lData);

    g_proc(hwnd, CPL_EXIT, 0L, 0L);
    
    free(g_CplBuf);
    FreeLibrary(g_InstCpl);
}


/*/////////////////////////////////////////////////////////////////
Function:   EnumApplets()
Parameters: hwnd    Parent window handle
            pszCpl  Address to the Name of CPL File
            pszAplt Address to the Applet Tokens Returned
*//////////////////////////////////////////////////////////////////
int  EnumApplets(HWND hwnd, LPCSTR pszCpl, LPSTR pszAplt)
{         
    int i, iRet;
    if (iRet=InitCPLProc(hwnd, pszCpl)) return iRet;    

    pszAplt[0] =0;
    for (i=0; i<g_nCount; ++i)
    {
        if (!g_CplBuf[i].szName[0]) // if no name for that Applet
            strcat(pszAplt, "Anonymous");
        else
            strcat(pszAplt, g_CplBuf[i].szName);
        strcat(pszAplt, ",");    // form token by token
    }

    pszAplt[strlen(pszAplt)-1] =0; // erase last ','

    CloseCPL(hwnd);
    return ECAERR_NOERROR;
}

/*/////////////////////////////////////////////////////////////////
Function:   CallAppletByIndex()
Parameters: hwnd    Parent window handle
            pszCpl  Address to the Name of CPL File
            idx     Index of the Applet
*//////////////////////////////////////////////////////////////////
int  CallAppletByIndex(HWND hwnd, LPCSTR pszCpl, int idx)
{
    int iRet;
    if (iRet=InitCPLProc(hwnd, pszCpl)) return iRet;    

    g_proc(hwnd, CPL_DBLCLK, (LONG)idx, g_CplBuf[idx].lData);

    CloseCPL(hwnd);
    return ECAERR_NOERROR;
}

/*/////////////////////////////////////////////////////////////////
Function:   CallAppletByName()
Parameters: hwnd    Parent window handle
            pszCpl  Address to the Name of CPL File
            pszAp   Address to the Name of the Applet
*//////////////////////////////////////////////////////////////////
int  CallAppletByName(HWND hwnd, LPCSTR pszCpl, LPCSTR pszAp)
{
    int i, iRet;
    if (iRet=InitCPLProc(hwnd, pszCpl)) return iRet;    

    for (i=0; i<g_nCount; ++i)
        if (!strcmp(g_CplBuf[i].szName, pszAp)) break; // found

    if (i<g_nCount) 
        g_proc(hwnd, CPL_DBLCLK, (LONG)i, g_CplBuf[i].lData);

    CloseCPL(hwnd);
    return i<g_nCount ? ECAERR_NOERROR: ECAERR_NOAPFOUND;
}


