/*------------------------------------------------------------------
EnumCpl.CPP:    Implementation of CEnumCplDlg class
Author:         Zuoliu Ding
------------------------------------------------------------------*/
#include <afxwin.h>
#include "resource.h"
#include "enumcpl.h"
#include "cplfnc.h"

#ifndef WIN32
#include <dos.h>
#endif
                                         
BEGIN_MESSAGE_MAP(CEnumCplDlg, CDialog)
    //{{AFX_MSG_MAP(CEnumCplDlg)
    ON_LBN_SELCHANGE(IDC_LISTCPL, OnSelchangeListCpl)
    ON_LBN_DBLCLK(IDC_LISTAPPLET, OnDblclkListApplet)
    //}}AFX_MSG_MAP
END_MESSAGE_MAP()

CEnumCplDlg::CEnumCplDlg(LPCSTR pCpl) : CDialog(CEnumCplDlg::IDD)
{
    //{{AFX_DATA_INIT(CEnumCplDlg)
    m_iRadio = 0;
    //}}AFX_DATA_INIT
    strcpy(m_szCpl, pCpl);
}

void CEnumCplDlg::DoDataExchange(CDataExchange* pDX)
{
    CDialog::DoDataExchange(pDX);
    //{{AFX_DATA_MAP(CEnumCplDlg)
    DDX_Control(pDX, IDC_LISTCPL, m_lstCpls);
    DDX_Control(pDX, IDC_LISTAPPLET, m_lstApps);
    DDX_Radio(pDX, IDC_RADIOINDX, m_iRadio);
    //}}AFX_DATA_MAP
}

/*/////////////////////////////////////////////////////////////////
Function:    OnInitDialog()
Note:        Find All CPL Files under System Dir in CPL List
*//////////////////////////////////////////////////////////////////
BOOL CEnumCplDlg::OnInitDialog() 
{
    CDialog::OnInitDialog();  
    CenterWindow();     
    
    char szPath[_MAX_PATH];
    if (!GetSystemDirectory(szPath, _MAX_PATH)) return FALSE;
    strcat(szPath, "\\*.cpl");

#ifdef WIN32 
    HANDLE           hFind;
    WIN32_FIND_DATA  findData;   
    if ((hFind=FindFirstFile(szPath, &findData)) 
                  ==INVALID_HANDLE_VALUE) return FALSE;

    do m_lstCpls.AddString(strlwr(findData.cFileName));
    while (FindNextFile(hFind, &findData));

    if (!FindClose(hFind)) return FALSE;  
    
#else  // for 16-bit
    struct _find_t  c_file;     
    if (_dos_findfirst(szPath, _A_NORMAL, &c_file)) return FALSE;

    do m_lstCpls.AddString(_strlwr(c_file.name));
    while (!_dos_findnext( &c_file ));
#endif

    m_lstCpls.SelectString(-1, m_szCpl);// init CPL List selection
    OnSelchangeListCpl();               // init Applet List content
    return TRUE;  
}

/*/////////////////////////////////////////////////////////////////
Function:    OnSelchangeListCpl()
Note:        Message Handler for Selecting a CPL File in CPL List
*//////////////////////////////////////////////////////////////////
void CEnumCplDlg::OnSelchangeListCpl() 
{
    int iSel;
    if (LB_ERR == (iSel=m_lstCpls.GetCurSel())) return;
    
    CString str;
    m_lstCpls.GetText(iSel, str);
    GetDlgItem(IDC_STATICAPPLAT)
        ->SetWindowText("Applets in " +str +":");
    m_lstApps.ResetContent();

    char szApplets[1024];
    UINT iRet;
    if (iRet=EnumApplets(GetSafeHwnd(), str, szApplets)) 
    {
#ifdef WIN32 
        if (iRet==ERROR_GEN_FAILURE) // possible 16-bit CPL file
        {
            iRet = WinExec("EnumCp16.EXE " +str, SW_SHOW); //switch  
            if (iRet>=HINSTANCE_ERROR) // invoke OK
            {
                CDialog::OnCancel(); // close current 32-bit EnumCpl
                return;
            }
            else
                AfxMessageBox("Err(31): A device not functioning."
                    "\nAnd Can't Execute EnumCp16 for " +str);
        }

#else  // for 16-bit
        if (iRet==ECAERR_NEED32BIT) // meet 32-bit CPL file
        {
            iRet = WinExec("EnumCpl.EXE " +str, SW_SHOW); //switch 
            if (iRet>=(int)HINSTANCE_ERROR) // invoke OK
            {
                CDialog::OnCancel(); // close current EnumCp16
                return;
            }
            else
                AfxMessageBox("Can't Execute EnumCpl for " +str);
        }
#endif

        m_lstApps.AddString("Can't Access !"); // other Err in Enum
        return;
    }

    // no Err, iRet==ECAERR_NOERROR(0) 
    LPTSTR pApplet =strtok(szApplets, _T(",")); 
    if (pApplet)        
    {
       do m_lstApps.AddString(pApplet);       // fill applet names
       while (pApplet=strtok(NULL, _T(","))); // get next if any   
    }
}

/*/////////////////////////////////////////////////////////////////
Function:    OnDblclkListApplet()
Note:        Message Handler for Calling an Applet in Applet List
*//////////////////////////////////////////////////////////////////
void CEnumCplDlg::OnDblclkListApplet() 
{
    CString strCpl, strAp;
    int     iSel;
    if (LB_ERR == (iSel=m_lstCpls.GetCurSel())) return;
    m_lstCpls.GetText(iSel, strCpl);
    
    if (LB_ERR == (iSel=m_lstApps.GetCurSel())) return;
    m_lstApps.GetText(iSel, strAp);
    if (strAp =="Can't Access !") return;

    UpdateData();  // get m_iRadio

    if (m_iRadio==0 && 
        CallAppletByIndex(/*NULL*/GetSafeHwnd(), strCpl, iSel)) 
    {
        AfxMessageBox("Can't Invoke this Applet by Index !");
        return;
    }

    if (m_iRadio==1 && 
        CallAppletByName(/*NULL*/GetSafeHwnd(), strCpl, strAp)) 
    {
        AfxMessageBox("Can't Invoke this Applet by Name !");
    }
}

