#include <windows.h>
#include <windowsx.h>
#include <tchar.h>
#include <assert.h>
#include "hclib.h"
#include "wdjsub.h"

HKEY openKey( LPCTSTR pszKey, REGSAM samDesired ) {
    HKEY hk = 0;
    long lResult = 0;
    TCHAR szKey[ MAX_PATH ] = { 0 };
    assert( 0 != pszKey && 0 != *pszKey );
    wsprintf( szKey, _T( "Software\\%s" ), pszKey );
    assert( KEY_READ == samDesired || KEY_WRITE == samDesired );
    lResult = RegOpenKeyEx( 
        HKEY_CURRENT_USER, szKey, 0, samDesired, &hk );
    if ( NOERROR != lResult && KEY_WRITE == samDesired ) {
        DWORD dwDisposition = 0;
        lResult = RegCreateKeyEx( HKEY_CURRENT_USER, szKey, 0, 0, 
            REG_OPTION_NON_VOLATILE, KEY_WRITE, 0,
            &hk, &dwDisposition );
    }
    return NOERROR == lResult ? hk : 0;
}

static int findStringExact( HWND hwndCombo, LPCTSTR pszSearch ) {
    int nPrevIndex = -1;
    int nIndex     = -1;
    
    while ( 0 <= (nIndex = 
        ComboBox_FindStringExact( hwndCombo, nIndex, pszSearch ) ) )
    {
        int nBufLength = 
            1 + ComboBox_GetLBTextLen( hwndCombo, nIndex );
        LPTSTR psz = malloc( nBufLength * sizeof( TCHAR ) );
        BOOL bMatch;

        ComboBox_GetLBText( hwndCombo, nIndex, psz );
        bMatch = 0 == _tcscmp( psz, pszSearch );
        free( psz );
        if ( bMatch ) {
            break; //*** LOOP EXIT POINT
        }
        if ( nIndex <= nPrevIndex ) { // Check for wrap-around
            nIndex = -1;
            break; //*** LOOP EXIT POINT
        }
        nPrevIndex = nIndex;
    }
    
    return nIndex;
}

static LRESULT WINAPI cbWndProc(
    HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    if ( WM_DESTROY == msg ) {
        histExit( hwnd );
    }
    return wdjCallOldProc( cbWndProc, hwnd, msg, wParam, lParam );
}

void WINAPI histInit( HWND hwndCombo, const HIST_DESCR *pDescr ) {
    assert( IsWindow( hwndCombo ) );
    wdjSubclass( cbWndProc, hwndCombo, (void *) pDescr );
    if ( 0 != pDescr && 0 != pDescr->_pszRegKey ) {
        HKEY hk = openKey( pDescr->_pszRegKey, KEY_READ );
        if ( 0 != hk ) {
            int iVal;
            for ( iVal = 0; iVal < pDescr->_nMaxEntries; ++iVal ) {
                TCHAR szName [ MAX_PATH ] = { 0 };
                TCHAR szValue[ MAX_PATH ] = { 0 };
                DWORD dwSize = sizeof szValue; // Bytes, not chars.
                DWORD dwType = 0;
                long lResult = NOERROR;

                wsprintf( szName, _T( "%s%d" ),
                    pDescr->_pszRegName, iVal );
                lResult = RegQueryValueEx( hk, szName, 0, 
                    &dwType, (BYTE *)( szValue ), &dwSize );
                if ( NOERROR == lResult && 0 != szValue[ 0 ] ) {
                    ComboBox_AddString( hwndCombo, szValue );
                }
            }
            RegCloseKey( hk );
        }
    }
}

static void limitEntries( HWND hwndCombo, int nLimitEntries ) {
    int nEntries = ComboBox_GetCount( hwndCombo );
    while ( nLimitEntries < nEntries ) {
        ComboBox_DeleteString( hwndCombo, --nEntries );
    }
}

void WINAPI histUpdate( HWND hwndCombo ) {
    const HIST_DESCR *pDescr = (const HIST_DESCR *)
        wdjGetData( cbWndProc, hwndCombo );
    const int nBufLength = 1 + ComboBox_GetTextLength( hwndCombo );
    if ( 1 < nBufLength ) {
        LPTSTR psz = malloc( nBufLength * sizeof( TCHAR ) );
        ComboBox_GetText( hwndCombo, 
            psz, nBufLength / sizeof( TCHAR ) );
        for ( ;; ) {
            int nIndex = pDescr->_isCaseSensitive
                ? findStringExact( hwndCombo, psz )
                : ComboBox_FindStringExact( hwndCombo, -1, psz );
            if ( nIndex < 0 ) {
                break; //*** LOOP EXIT POINT
            }
            ComboBox_DeleteString( hwndCombo, nIndex );
        }
        
        ComboBox_InsertString( hwndCombo, 0, psz );
        free( psz );
        ComboBox_SetCurSel( hwndCombo, 0 );
    }

    limitEntries( hwndCombo, pDescr->_nMaxEntries );
}

void WINAPI histExit( HWND hwndCombo ) {
    const HIST_DESCR *pDescr = (const HIST_DESCR *)
        wdjGetData( cbWndProc, hwndCombo );
    if ( 0 != pDescr && 0 != pDescr->_pszRegKey ) {
        HKEY hk = openKey( pDescr->_pszRegKey, KEY_WRITE );
        if ( 0 != hk ) {
            int nEntries = ComboBox_GetCount( hwndCombo );
            int iVal;
            for ( iVal = 0; iVal < pDescr->_nMaxEntries; ++iVal ) {
                TCHAR szName [ MAX_PATH ] = { 0 };
                TCHAR szValue[ MAX_PATH ] = { 0 };
                wsprintf( szName, _T( "%s%d" ),
                    pDescr->_pszRegName, iVal );
                if ( iVal < nEntries ) {
                    ComboBox_GetLBText( hwndCombo, iVal, szValue );
                }
                RegSetValueEx( 
                    hk, szName, 0, REG_SZ, (CONST BYTE *) szValue, 
                    (lstrlen( szValue ) + 1) * sizeof( TCHAR ) );
            }
            RegCloseKey( hk );
        }
    }
}
