#include "common.h"

static void gradient( HDC hdc, RECT *prc ) {
    TRIVERTEX triVertex[] = { { 0, 0, 0x0000, 0x7f00, 0x7f00 },
                              { 0, 0, 0x7f00, 0x0000, 0x7f00 }, };
    GRADIENT_RECT gradientRect = { 0, 1 };
    triVertex[ 0 ].x = prc->left;
    triVertex[ 0 ].y = prc->top;
    triVertex[ 1 ].x = prc->right;
    triVertex[ 1 ].y = prc->bottom;
    GradientFill( hdc, 
        triVertex, 2, &gradientRect, 1, GRADIENT_FILL_RECT_H );
}

static void onPaint( HWND hwnd ) {
    PAINTSTRUCT ps;
    RECT        rc;
    TCHAR       sz[ 1000 ] = { 0 };

    BeginPaint( hwnd, &ps );
    GetClientRect( hwnd, &rc );
    gradient( ps.hdc, &rc );
    InsetRect( &rc, BORDER, BORDER );
    GetWindowText( hwnd, sz, 1000 );
    SetTextColor( ps.hdc, RGB( 255, 255, 255 ) );
    SetBkMode( ps.hdc, TRANSPARENT );
    drawText( ps.hdc, sz, &rc, DT_CENTER );
    EndPaint( hwnd, &ps );
}

static LRESULT CALLBACK layeredWndProc(
    HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    switch ( msg ) {
    HANDLE_MSG( hwnd, WM_PAINT       , onPaint        );
    HANDLE_MSG( hwnd, WM_CHAR        , onChar         );
    HANDLE_MSG( hwnd, WM_NCHITTEST   , onNCHitTest    );
    }
    return DefWindowProc( hwnd, msg, wParam, lParam );
}

HWND initLayered( HINSTANCE hinst, BOOL bTranslucent ) {
    HWND     hwnd = 0;
    WNDCLASS wc   = { 0 };
    SIZE     size = { 0 };

    wc.hInstance     = hinst;
    wc.hCursor       = LoadCursor( 0, IDC_ARROW );
    wc.lpszClassName = _T( __FILE__ );
    wc.lpfnWndProc   = layeredWndProc;
    RegisterClass( &wc );

    windowSizeFromText( _T( "Windows 2000" ), &size );
    hwnd = CreateWindowEx(
        WS_EX_TOPMOST | WS_EX_LAYERED, _T( __FILE__ ), 
        _T( "Windows 2000" ), WS_POPUP, 
        100, bTranslucent ? 300 : 400, size.cx, size.cy,
        HWND_DESKTOP, 0, hinst, 0 );
    if ( bTranslucent ) {
        SetLayeredWindowAttributes( hwnd, 0, 127, LWA_ALPHA );
    } else {
        SetLayeredWindowAttributes( hwnd, 
            RGB( 255, 255, 255 ), 0, LWA_COLORKEY );
    }
    ShowWindow( hwnd, SW_SHOWNORMAL );
    UpdateWindow( hwnd );
    return hwnd;
}

/* end of file */
