// main.c: This program takes a fully qualified file name as the
//         only parameter. It will attempt to open that file,
//         requesting credentials if necessary to establish a 
//         connection.

#define UNICODE
#define _UNICODE

#include "windows.h"
#include "auth.h"

HINSTANCE hInst;

int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInst, 
                     LPSTR lpCmdLine, int nCmdShow)
{
    HANDLE  hFile;
    TCHAR   sz[MAX_PATH], szFile[MAX_PATH];
    int     argc;
    LPTSTR* pszCmdLine;

    // Validate command line args (only required parameter is fully
    // qualified path).
    pszCmdLine = CommandLineToArgvW(GetCommandLine(), &argc);

    if (argc < 2 || pszCmdLine == NULL) {
        MessageBox(NULL, TEXT("Usage: auth <path_and_filename>"),
                   TEXT("AUTH.EXE"), MB_OK);
        return ERROR_INVALID_PARAMETER;
    }

    lstrcpy(szFile, pszCmdLine[1]);

    // attempt to open the specified file

    hFile = CreateFile(szFile, GENERIC_READ, FILE_SHARE_READ, NULL,
                       OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);

    if (hFile == INVALID_HANDLE_VALUE) {

        switch (GetLastError()) {

        case ERROR_FILE_NOT_FOUND:
            wsprintf(sz, TEXT("Could not find file %s.\n"), 
                     szFile);
            MessageBox(NULL, sz, TEXT("AUTH.EXE"), MB_OK);
            return ERROR_FILE_NOT_FOUND;

        case ERROR_LOGON_FAILURE:
        case ERROR_PATH_NOT_FOUND:
            if (!AuthenticateUser(hInstance, szFile)) {
                wsprintf(sz, TEXT("ERROR: Can't open %s (%d).\n"),
                         szFile, GetLastError());
                MessageBox(NULL, sz, TEXT("AUTH.EXE"), MB_OK);
                return GetLastError();
            }
               
            hFile = CreateFile(szFile, GENERIC_READ,
                        FILE_SHARE_READ, NULL, OPEN_EXISTING,
                        FILE_ATTRIBUTE_NORMAL, NULL);
            if (hFile == INVALID_HANDLE_VALUE) {
                wsprintf(sz, TEXT("ERROR: Can't open %s (%d).\n"),
                         szFile, GetLastError());
                MessageBox(NULL, sz, TEXT("AUTH.EXE"), MB_OK);
                return GetLastError();
            }
            break;

        default:
            wsprintf(sz, TEXT("ERROR: Could not open %s (%d).\n"), 
                     szFile, GetLastError());
            MessageBox(NULL, sz, TEXT("AUTH.EXE"), MB_OK);
            return GetLastError();
        }
    }

    wsprintf(sz, TEXT("File %s successfully opened.\n"), szFile);
    MessageBox(NULL, sz, TEXT("AUTH.EXE"), MB_OK);

    CloseHandle(hFile);
    return ERROR_SUCCESS;

} // WinMain


