#include "common.h"

static HRGN getPathRgn( LPCTSTR psz, SIZE *pSize ) {
    RECT rc   = { 0 };
    HDC  hdc  = GetDC( 0 );
    HRGN hrgn = 0;

    drawText( hdc, psz, &rc, DT_CALCRECT | DT_CENTER );
    pSize->cx = rc.right - rc.left;
    pSize->cy = rc.bottom - rc.top;
    ReleaseDC( 0, hdc );

    hdc = CreateCompatibleDC( 0 );
    PatBlt( hdc, 0, 0, pSize->cx, pSize->cy, WHITENESS );
    BeginPath( hdc );
    drawText( hdc, psz, &rc, DT_CENTER );
    EndPath( hdc );
    hrgn = PathToRegion( hdc );
    DeleteDC( hdc );

    return hrgn;
}

static void onPaint( HWND hwnd ) {
    PAINTSTRUCT ps;
    RECT        rc;
    HBRUSH      hbr = CreateSolidBrush( RGB( 128, 0, 0 ) );
    BeginPaint( hwnd, &ps );
    GetClientRect( hwnd, &rc );
    FillRect( ps.hdc, &rc, hbr );
    EndPaint( hwnd, &ps );
    DeleteBrush( hbr );
}

static LRESULT CALLBACK wndProc(
    HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    switch ( msg ) {
    HANDLE_MSG( hwnd, WM_PAINT       , onPaint        );
    HANDLE_MSG( hwnd, WM_CHAR        , onChar         );
    HANDLE_MSG( hwnd, WM_NCHITTEST   , onNCHitTest    );
    }
    return DefWindowProc( hwnd, msg, wParam, lParam );
}

HWND initRgn( HINSTANCE hinst ) {
    WNDCLASS wc   = { 0 };
    SIZE     size = { 0 };
    HRGN     hrgn = 0;
    HWND     hwnd = 0;

    wc.hInstance     = hinst;
    wc.hCursor       = LoadCursor( 0, IDC_ARROW );
    wc.lpszClassName = _T( __FILE__ );
    wc.lpfnWndProc   = wndProc;
    RegisterClass( &wc );

    hrgn = getPathRgn( _T( "PathToRegion" ), &size );
    hwnd = CreateWindowEx(
        WS_EX_TOPMOST, _T( __FILE__ ), _T( "PathToRegion" ),
        WS_POPUP, 100, 200, size.cx, size.cy,
        HWND_DESKTOP, 0, hinst, 0 );
    SetWindowRgn( hwnd, hrgn, FALSE );
    ShowWindow( hwnd, SW_SHOWNORMAL );
    UpdateWindow( hwnd );
    
    return hwnd;
}

/* end of file */
