#include "common.h"

#define HANDLE_WM_EXITSIZEMOVE(hwnd, wParam, lParam, fn) \
    ((fn)(hwnd), 0L)

static void onPaint( HWND hwnd ) {
    int         x, y;
    PAINTSTRUCT ps;
    RECT        rc;
    HDC         hdcBmp;
    HBITMAP     hbmp, hbmpSaved;
    TCHAR       sz[ 1000 ] = { 0 };

    BeginPaint( hwnd, &ps );
    GetClientRect( hwnd, &rc );
    hbmp = CreateBitmap( rc.right, rc.bottom, 1, 1, 0 );
    hdcBmp = CreateCompatibleDC( ps.hdc );
    hbmpSaved = SelectBitmap( hdcBmp, hbmp );
    PatBlt( hdcBmp, 0, 0, rc.right, rc.bottom, WHITENESS );
    GetWindowText( hwnd, sz, 1000 );
    InsetRect( &rc, BORDER, BORDER );
    drawText( hdcBmp, sz, &rc, DT_CENTER );
    InflateRect( &rc, BORDER, BORDER );

    for ( y = rc.top; y < rc.bottom; ++y ) {
        for ( x = rc.left; x < rc.right; ++x ) {
            const COLORREF cr = GetPixel( ps.hdc, x, y );
            int r = MulDiv( GetRValue( cr ), 127, 255 );
            int g = MulDiv( GetGValue( cr ), 127, 255 );
            if ( 0 != GetPixel( hdcBmp, x, y ) ) {
                r += 128;
                g += 128;
            }
            SetPixelV( ps.hdc, x, y, RGB( r, g, 128 ) );
        }
    }

    SelectBitmap( hdcBmp, hbmpSaved );
    DeleteDC( hdcBmp );
    DeleteBitmap( hbmp );
    EndPaint( hwnd, &ps );
}

static void onExitSizeMove( HWND hwnd ) {
    HWND hwndOther;
    ShowCursor( FALSE );
    ShowWindow( hwnd, SW_HIDE );
    for ( hwndOther = GetTopWindow( GetDesktopWindow() );
          IsWindow( hwndOther );
          hwndOther = GetNextWindow( hwndOther, GW_HWNDNEXT ) )
    {
        if ( hwnd != hwndOther ) {
            InvalidateRect( hwndOther, 0, 0 );
            UpdateWindow( hwndOther );
        }
    }
    ShowWindow( hwnd, SW_SHOW );
    UpdateWindow( hwnd );
    ShowCursor( TRUE );
}

static LRESULT CALLBACK regionWndProc(
    HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam )
{
    switch ( msg ) {
    HANDLE_MSG( hwnd, WM_PAINT       , onPaint        );
    HANDLE_MSG( hwnd, WM_CHAR        , onChar         );
    HANDLE_MSG( hwnd, WM_NCHITTEST   , onNCHitTest    );
    HANDLE_MSG( hwnd, WM_EXITSIZEMOVE, onExitSizeMove );
    }
    return DefWindowProc( hwnd, msg, wParam, lParam );
}

HWND init95( HINSTANCE hinst ) {
    WNDCLASS wc   = { 0 };
    SIZE     size = { 0 };
    HWND     hwnd = 0;

    wc.hInstance     = hinst;
    wc.hCursor       = LoadCursor( 0, IDC_ARROW );
    wc.lpszClassName = _T( __FILE__ );
    wc.lpfnWndProc   = regionWndProc;
    RegisterClass( &wc );
    windowSizeFromText( _T( "Windows 95" ), &size );
    hwnd = CreateWindowEx(
        WS_EX_TOPMOST, _T( __FILE__ ), _T( "Windows 95" ),
        WS_POPUP, 100, 100, size.cx, size.cy,
        HWND_DESKTOP, 0, hinst, 0 );
    ShowWindow( hwnd, SW_SHOWNORMAL );
    UpdateWindow( hwnd );
    return hwnd;
}

/* end of file */
